/*
 * gen-ioctl: Ioctl interface for generic ioctl calls
 * 
 * Copyright (C) 2005 TiVo Inc. All Rights Reserved.
 */

#include <linux/init.h>
#include <linux/mm.h>
#include <linux/fs.h>
#include <linux/module.h>
#include <asm/uaccess.h>
#include <linux/slab.h>
#include <linux/tivo-genioctl.h>
#include <linux/tivo-crashallassert.h>
#include <linux/sgat-dio.h>
#include <asm/scatterlist.h>

#ifdef CONFIG_KSTACK_GUARD
extern void test_kstack_guard_check(struct task_struct *p);
#endif

#define GIOCTL_MAJOR 243

int gioctl_open (struct inode *inode, struct file *filp){
        return 0;
}

int gioctl_ioctl (struct inode *inode, struct file *file, 
                unsigned int cmd, unsigned long arg) {

        if ( !arg ){
                printk(KERN_ERR "[gen_ioctl] Invalid arg\n");
                return -EINVAL;
        }
        if(_IOC_TYPE(cmd) != GIOCTL_IOC_MAGIC)
        {
                printk(KERN_ERR "[gen_ioctl] cmd (0x%x) mismatch, "
                                "called with 0x%x while expecting 0x%x\n", 
                                cmd, _IOC_TYPE(cmd), GIOCTL_IOC_MAGIC);
                return -EINVAL;
        }

        switch(cmd){
                case CRASHALLASSRT:
                        return tivocrashallassert(arg);
                break;
#ifdef CONFIG_TICK_KILLER
		case TICK_KILLER:
			return tick_killer_start(arg);
		break;
#endif
                case UPDATE_EXEC_CTX:
                    /*Ineffective ioctl. Handling for the backward compatibility purposes.
                     * Has be removed.*/
                break;
#ifdef CONFIG_KSTACK_GUARD
                case TEST_KSTACK_GUARD:
                        test_kstack_guard_check(current);
                        return 0;
                break;
#endif
                case TEST_SGAT_DIO:
                {
                        struct sgat_dio_fops gfops;
                        sgat_dio_req dio_req;
                        int c;
                        int k = 'z';

                        sgat_dio_get_default_fops(&gfops);
                        
                        if ( sgat_dio_init_req(&gfops,&dio_req,(sgat_dio_hdr_t*)arg)){
                                printk("Error: sgat_dio_init_req\n");
                                return -EINVAL;
                        }

                        if ( sgat_dio_build_sg_list(&gfops, &dio_req, 1)){
                                printk("Error: sgat_dio_build_sg_list\n");
                                return -EINVAL;
                        }

                        struct  scatterlist *p = (struct  scatterlist *) dio_req.sclist;
                        for ( c =0; c < dio_req.k_useg; c++){
                            memset(p->address,k,p->length);
                            p++;
                        }
                        sgat_dio_exit_req(&gfops,&dio_req,1);
                }
                return 0;
                break;
                default:
                        printk(KERN_ERR "gen_ioctl: Invalid command\n");
                        return -EINVAL;
        }
       return -EINVAL; 
}

static struct file_operations gioctl_fops = {
        ioctl:   gioctl_ioctl,
        open:    gioctl_open,
};

void __init init_gioctl_if(void){
        int result;
        result = register_chrdev(GIOCTL_MAJOR, "gioctl", &gioctl_fops);
        if ( result < 0 )
                printk(KERN_ERR "gioctl: "
                                "failed to register char dev [major %d]\n",GIOCTL_MAJOR);
}

module_init(init_gioctl_if);
